import bpy
from random import random
from bpy.props import StringProperty,IntProperty,EnumProperty
from bl_operators.wm import WM_OT_doc_view
from speaker_tools.Equalizer import showEqualiser , getAction
def dprint(str):
    DEBUG = False
    if bpy.app.debug or DEBUG:
        print(str)
        
def finddriver(self,context):
    channel = self.channel
    driver = None
    dprint("CONTEXT %s"%context.space_data.type)
    #tempted to ust bpy.data.objects meshes ...
    search_list = [context.scene]
    try:
        if context.world is not None:
            search_list.append(context.world)
    except:
        dprint("NO CONTEXT WORLD") 

    if context.object is not None:
        search_list.append(context.object)
        if context.object.data is not None:
            search_list.append(context.object.data)
        for mat in context.object.material_slots:
            search_list.append(mat)
    for search in search_list: 
        #search list could be extended for texture_slots[...].texture etc
        try:
            
            driver = search.animation_data.drivers[-1]
            dprint("DRIVER TYPE %s %s  %s"%(driver.id_data,driver.data_path,context.window_manager.clipboard))
            dprint(driver.select)
            if driver.select and driver.data_path.find(context.window_manager.clipboard) > -1:
                print("return %s.%s"%(search.name,driver.data_path))    
                return(driver.driver)
            
        except:
            continue
    bpy.ops.anim.driver_button_remove(all=False)
    self.report({'WARNING'},"No Driver found ... REMOVED")
    dprint("No driver found")
    return(None) 

   
def main(self,context):
    if context is None:
        return False
    #ugly way to see if we are adding new driver or editing custom driver
    space = context.space_data
    search = True
    if context.object is None:
       if space.use_pin_id:
           speaker = space.pin_id
    else:
       speaker = context.object.data

    channel = self.channel
    dprint('.....Datapath %s channel %s'%(bpy.context.window_manager.clipboard,channel))
    SVP = bpy.types.SoundVisualiserPanel
    drivername = "Driver%d"%SVP.driver_index
    if drivername in speaker: # and not context.speaker:
        driver = speaker.animation_data.drivers[SVP.driver_index].driver
    else:
        driver = finddriver(self,context)
    if driver:
        var = driver.variables[0]
        var.type = "SINGLE_PROP"
        var.name = channel
        target = var.targets[0]
        target.id_type = "SPEAKER"
        target.id = bpy.types.Scene.context_speaker.id_data
        target.data_path = '["%s"]'%channel
        driver.expression = 'SoundDrive(%s)'%channel

class AddCustomSoundDriverToChannel(bpy.types.Operator):
    '''Add Custom Sound Drivers'''
    bl_idname = "speaker.add_driver_channel"
    bl_label = "Add Custom Sound Driver"
    channel = StringProperty(default="Menu")
    contextspeakername = StringProperty(default="None")
    driver_index = IntProperty(default=-1)  # if this is passed we know the driver is the context speaker.
    @classmethod
    def poll(cls, context):
        return context.active_object is not None
    
    def invoke(self, context, event):
        if self.contextspeakername != "None":
           setcontextspeaker(self.contextspeakername)
           return {'RUNNING_MODAL'}
        space = context.space_data

        if self.driver_index != -1:
            bpy.types.Scene.context_speaker = context.speaker
            bpy.types.SoundVisualiserPanel.driver_index = self.driver_index
        if self.channel == "Menu":
            wm = context.window_manager
            #return wm.invoke_props_dialog(self)           
            return wm.invoke_popup(self)
        else:
            #main(self,context)
            return self.execute(context)
            #return{'PASS_THROUGH'}

    def draw(self,context):
        layout = self.layout
        if context.scene.context_speaker is None:
            layout.label("NO Speaker Selected",icon="ERROR")
            layout.menu("speaker.contextspeaker")
        else:
            layout.label("Select Channel")

        sce = context.scene
        speaker = bpy.types.Scene.context_speaker
        if self.driver_index > -1:
            #layout.label("driverindex %d"%self.driver_index)
            driver = speaker.animation_data.drivers[0].driver
            layout.prop(driver,"expression")
        else:
            driver = finddriver(self,context)
            if driver is not None:
                print(driver)
                layout.prop(driver,"expression")

        action = None
        if speaker and speaker.animation_data:

            if speaker.animation_data.use_nla and len(speaker.animation_data.nla_tracks) > 0:
                action = speaker.animation_data.nla_tracks[0].strips[0].action
            else:
                action = speaker.animation_data.action
                if action:
                    layout.menu("soundtest.menu",text=action.name,icon="SOUND")
        
        if action: 
            
            box = layout.box()
            box.enabled = context.scene.context_speaker is not None
            
            i = 0
            minf = -1
            maxf = -1
            name = "Channel"
            if "Channels" in action:
                channels = action["Channels"]
                name = action["channel_name"]

                minf = action["minf"]
                maxf = action["maxf"]
                
            else:
                channels = len(action.fcurves)
            

            COLS = 4
            #box.split(percentage=0.50)
            while i < channels:
                channel = '%s%d'%(name,i)
                rna = speaker["_RNA_UI"][channel]
                if not i % COLS : row = layout.row()
                col = row.column()
                col.operator("speaker.add_driver_channel",text="%d"%i).channel=channel
                col.enabled = not (float(rna["b"] - rna["a"]) < 0.0001)
                i += 1         
        #layout.menu("sounds.channelmenu")
           
    def execute(self, context):
        dprint("execute %s"%self.channel)
        if self.channel != 'Menu':
            main(self,context)
        return {'FINISHED'}

class SimpleOperator(bpy.types.Operator):
    '''Add Custom Sound Drivers'''
    bl_idname = "wm.doc_view"
    bl_label = "Add Custom Sound Driver"
    doc_id = StringProperty(default="")
    _prefix = ""

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def execute(self, context):
        if self.doc_id != "":
            dprint("Overwriting view docs for %s"%self.doc_id)
            #return { 'CANCELLED' }
        bpy.ops.ui.copy_data_path_button()
        if bpy.ops.anim.copy_driver_button.poll():
            dprint("This already has a driver")
        bpy.ops.anim.driver_button_remove(all=False)
        bpy.ops.anim.driver_button_add(all=False)
        return(bpy.ops.speaker.add_driver_channel('INVOKE_DEFAULT'))
        


class ContextSpeakerSelectMenu(bpy.types.Menu):
    bl_idname = "speaker.select_contextspeaker"
    bl_label = "Choose speaker to drive"
    driver = None
    
    def draw(self, context):
        speakers = [speaker for speaker in bpy.data.speakers if "vismode" in speaker]
        if bpy.types.Scene.context_speaker is not None:
            self.layout.operator("speaker.add_driver_channel",text="OFF").contextspeakername = "TURNOFF"
        for speaker in speakers:
            self.layout.operator("speaker.add_driver_channel",text="%s (%s)"%(speaker.name,speaker.sound.name)).contextspeakername = speaker.name


class ContextSpeakerMenu(bpy.types.Menu):
    bl_idname = "speaker.contextspeaker"
    bl_label = "Choose speaker to drive"
    driver = None
    
    def draw(self, context):
        speakers = [speaker for speaker in bpy.data.speakers if "Channel0" in speaker]
        for speaker in speakers:
            self.layout.operator("speaker.add_driver_channel",text="%s (%s)"%(speaker.name,speaker.sound.name)).contextspeakername = speaker.name



def setcontextspeaker(name):
    speaker = None
    if name in bpy.data.speakers:
        speaker = bpy.data.speakers[name]
        if bpy.types.Scene.context_speaker is None:
            overwriteviewdoc(True)
    elif name == "TURNOFF":
        overwriteviewdoc(False)
    bpy.types.Scene.context_speaker = speaker



def overwriteviewdoc(enable):
    try:
        if enable:
            bpy.utils.register_class(SimpleOperator)
            bpy.utils.unregister_class(WM_OT_doc_view)
        else:
            bpy.utils.unregister_class(SimpleOperator)
            bpy.utils.register_class(WM_OT_doc_view)
    except:
        dprint("HUMFFF this is driving me nuts")
        
bpy.types.Speaker.rightclick = EnumProperty(items=(
            ("OFF","OFF","Prop toolbox right click off"),
            ("ON","ON","Overwrite view docs operator"),
            ),
            name="Right Click Fudge",
            default = "OFF",
            description = "Add speaker drivers from right click toolbox",
            #update=overwriteviewdoc,
            options= { "HIDDEN" }
            
            )


class SoundToolPanel(bpy.types.Panel):


    bl_label = "Sound Tools"
    bl_space_type = "VIEW_3D"
    #bl_region_type = "UI"
    bl_region_type = "TOOLS"

    
    @classmethod
    def poll(cls, context):

        speakers = [speaker for speaker in bpy.data.speakers if "vismode" in speaker]
        return len(speakers) > 0
    
    
    def draw(self, context):
        layout = self.layout
        row = layout.row()
        context_speaker = bpy.types.Scene.context_speaker
        if context_speaker is not None:
            row.menu("speaker.select_contextspeaker",text="%s (%s)"%(context_speaker.name,context_speaker.sound.name))
            row = layout.row()
            box = row.box()
            action = getAction(context_speaker)
            if action:
                showEqualiser(box,context_speaker,action,info=False)
        else:
            row.menu("speaker.select_contextspeaker")
            row = layout.row()
            box = row.box()
            box.label("Drive using the view docs operator",icon="INFO")
            



def register():
    bpy.types.Scene.context_speaker = None
    #bpy.utils.register_class(SimpleOperator)
    bpy.utils.register_class(ContextSpeakerSelectMenu)
    bpy.utils.register_class(AddCustomSoundDriverToChannel)
    bpy.utils.register_class(ContextSpeakerMenu)
    bpy.utils.register_class(SoundToolPanel)

def unregister():
    bpy.types.Scene.context_speaker = None
    #bpy.utils.unregister_class(SimpleOperator)
    bpy.utils.unregister_class(AddCustomSoundDriverToChannel)
    bpy.utils.unregister_class(ContextSpeakerMenu)
    bpy.utils.unregister_class(ContextSpeakerSelectMenu)
    bpy.utils.unregister_class(SoundToolPanel)
