import bpy
from bpy.props import *
from math import log
from speaker_tools.NLALipsync import SoundTools_LipSync_PT
from bl_ui.properties_data_speaker import DATA_PT_context_speaker,DATA_PT_speaker,DATA_PT_cone,DATA_PT_distance,DATA_PT_custom_props_speaker


from bpy.app.handlers import persistent
# add drivers to the namespace

@persistent
def InitSoundTools(scene):
    print("LOAD CUSTOM DRIVERS")
    bpy.app.driver_namespace["SoundDrive"] = SoundDrive
    reset_drivers()
    #DATA_PT_context_speaker.append(SoundVisualiserPanel.drawvisenum)
    #DATA_PT_context_speaker.append(SoundVisualiserPanel.draw)

    #make the speaker live to the panel

    bpy.app.handlers.frame_change_pre.append(live_speaker_view)
# method to reinvigorate disabled drivers in file.
def reset_drivers():
    collections = ["scenes","objects","meshes","materials","textures","speakers","worlds","curves","armatures","particles","lattices","shape_keys","lamps","cameras"]

    for col in collections:
        collection = eval("bpy.data.%s"%col)
        for ob in collection:
            if ob.animation_data is not None:
                for driver in ob.animation_data.drivers:
                    driver.driver.expression = driver.driver.expression
def getAction(speaker):
    action = None
    if speaker.animation_data:

        if len(speaker.animation_data.nla_tracks) > 0:
            nla = True
            action = speaker.animation_data.nla_tracks[0].strips[0].action                
        else:
            action = speaker.animation_data.action
    return action
                                       


#FRAME change method to make the equalizer update live to panel

def live_speaker_view(scene):
    
    #print("Frame Change", scene.frame_current)
    frame = scene.frame_current
    speakers = [speaker for speaker in bpy.data.speakers if "Channel0" in speaker]
    for speaker in speakers:
        # setting the enum to the enum will make the panel live
        speaker.vismode = speaker.vismode
        if speaker.animation_data:
            action = speaker.animation_data.action
            if action:
                timemarkers = [marker.name for marker in action.pose_markers if marker.frame == frame]
                if len(timemarkers):
                    speaker.papagayo_phonemes = timemarkers[0]
                    





# DRIVER methods ######################################
def SoundDrive(channel,amp=1.0,norm=1.0):
    #print("SoundDrive %s"%channel)
    return(amp*norm*channel)


def getrange(fcurve,tolerance):
    #get the minimum and maximum points from an fcurve
    
    REJECT = tolerance #reject frequencty
    #print(len(fcurve.sampled_points))
    points = [0.0]
    for point in fcurve.sampled_points:
        # bake sound to fcurve returns some rubbish when the frequency is out of range
        # often the only usable point is in the first 
        if point.co[1] > -REJECT and point.co[1] < REJECT :# print("PROBABLE FECK UP")
            points.append(point.co[1])
    #print(points)
    #print("GETRANGE",min(points),max(points))
    hasrange =  abs(max(points)-min(points)) > 0.01
    if hasrange:
        return True,min(points),max(points)
    else:
        return False,0.0,0.0

def f(freq):
    #output a format in Hz or kHz
    
    if freq < 1000:
        return("%dHz"%freq)
    elif freq < 1000000:
        khz = freq/1000
        
        return("%.2fkHz"%khz)
    return(" ")
    
def fetch(collection,name,create=False,**newargs):
# returns item from collection , None if not there, creates if create is True passes newargs to new method.
    
    if name in collection:
        return collection[name]
    
# maybe put in code for replace too.
    
    if create:
        if len(newargs):
            try:
                
                newitem = collection.new(**newargs)
                newitem.name = name
            except:
                newitem = collection.new(name=name,**newargs)
        else:
            try:
                newitem = collection.new(name=name) #for things like scene objects
            except:
                newItem = None
        
        return newitem    
    return None


class soundtest():
    speaker = None
    
    def __init__(self,speaker=None):
        if speaker:
            self.speaker=speaker
    
class DataButtonsPanel():
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = "data"

    @classmethod
    def poll(cls, context):
        return context.speaker is not None 

class SoundVisualiserPanel(DataButtonsPanel, bpy.types.Panel):
    bl_label = " "
    bl_options = {'HIDE_HEADER'}
    #bl_space_type = "GRAPH_EDITOR"
    #bl_region_type = "UI"
    driver_index = 0
    def draw_header(self, context):
        layout = self.layout
        row = layout.row()
        layout.menu("soundtest.menu",text="Select Sound Action",icon="SOUND")

    def drawvisenum(self,context):
        speaker = context.speaker
        layout = self.layout
        row = layout.row()
        row.prop(speaker,'vismode',expand=True)

    def drawcontextspeaker(self,context,layout):

        ob = context.object
        speaker = context.speaker
        space = context.space_data

        split = layout.split(percentage=0.65)

        if ob:
            split.template_ID(ob, "data")
        elif speaker:
            split.template_ID(space, "pin_id")

        
    def draw(self, context):
        sce = context.scene
        if context.space_data.use_pin_id:
            speaker = context.space_data.pin_id
        else:
            speaker = context.object.data
        layout = self.layout
        nla = False
        action = None
        frame = sce.frame_current
        self.drawvisenum(context)
        row = layout.row()
        self.drawcontextspeaker(context,row)
        has_sound = (speaker.sound is not None)
        box = layout.box()
        if not has_sound:
            row = box.row()
            row.template_ID(speaker, "sound", open="sound.open_mono")
            return
        
        action = getAction(speaker)

        '''
        if "Channels" not in action:

            row = box.row()
            row.template_ID(speaker.animation_data,"action")
        '''   
        row = box.row(align=True)
        #row.alignment = "E"
        if has_sound:
            col = row.column()
            col.alignment="LEFT"
            col.menu("soundtest.menu",text="",icon="SOUND")
        #col.alignment="LEFT"
        if action: 
            col = row.column()
            col.prop(action,"name",emboss=True,text="")
        else: 
            col = row.column()
            #col.label("NO SOUND BAKED")
            col.operator("speaker.visualise",text="Bake %s"%speaker.sound.name)
        col = row.column()
        col.alignment="RIGHT"
        split = col.split()
        split.alignment="RIGHT"
        if not has_sound:
            return
        if action: split.prop(action,"use_fake_user",toggle=True,text="F",emboss=True)
        split.operator("speaker.visualise",text="",icon="ZOOMIN")




        if speaker.vismode == "SOUND":
            box.label("Sound",icon="FILE_SOUND")
            soundbox = box.box()
            row = soundbox.row()
            split = row.split(percentage=0.75)
            split.template_ID(speaker, "sound", open="sound.open_mono")
            split.prop(speaker, "muted")
            row = soundbox.row()
            row.prop(speaker, "volume")
            row.prop(speaker, "pitch")

            box.label("Distance",icon="ARROW_LEFTRIGHT")
            distancebox = box.box()
            split = distancebox.split()

            col = split.column()
            col.label("Volume:")
            col.prop(speaker, "volume_min", text="Minimum")
            col.prop(speaker, "volume_max", text="Maximum")
            col.prop(speaker, "attenuation")
    
            col = split.column()
            col.label("Distance:")
            col.prop(speaker, "distance_max", text="Maximum")
            col.prop(speaker, "distance_reference", text="Reference")

            box.label("Cone",icon="MESH_CONE")
            conebox = box.box()
            split = conebox.split()
            col = split.column()
    
            col.label("Angle:")
            col.prop(speaker, "cone_angle_outer", text="Outer")
            col.prop(speaker, "cone_angle_inner", text="Inner")

            col = split.column()

            col.label("Volume:")
            col.prop(speaker, "cone_volume_outer", text="Outer")

        if action:  
            minf = -1
            maxf = -1
            scale = action["row_height"]
            name = "Channel"
            if "Channels" in action:
                channels = action["Channels"]
                name = channel_name = action["channel_name"]
                minf = action["minf"]
                maxf = action["maxf"]
                start = action["start"]
                end = action["end"]
                
            else:
                channels = len(action.fcurves)
                minf = 0.0
                maxf = 0.0
                start = 0
                end = channels 


            if start >= end:
                end = start + 1
            i = start 
                

            if speaker.vismode == "VISUAL":
                showEqualiser(box,speaker,action)

            elif speaker.vismode == "DRIVERS":
                box = layout.box()
                row = box.row()
                driver_index = bpy.types.SoundVisualiserPanel.driver_index
                #row.enabled = False  # SINGLE
                # SINGLE will support multiple copyable drivers one for now.
                driver = speaker.animation_data.drivers[driver_index]
                row.menu("soundtest.menu",text="%s %s"%(driver.data_path,driver.driver.expression),icon="DRIVER")    
                for i,driver in enumerate(speaker.animation_data.drivers):
                    if i != driver_index:
                        continue
                    box = layout.box()
                    row = box.row()
                    row.prop(speaker,driver.data_path,slider=True)
                    row = box.row()
                    row.prop(driver.driver,"expression")
                    
                    
                    for var in driver.driver.variables:
                        row = box.row()
                        row.prop(var,"name")
                        row = box.row()
                        row.prop(var.targets[0],"data_path")
                    box.operator("speaker.add_driver_channel",text="Rechoose target freq").driver_index=i
            elif speaker.vismode == "LIPSYNC":
                bpy.types.SoundTools_LipSync_PT.draw(self,context)
            #print("ACTION",action)
            #timemarkers = ['rest']
            timemarkers = [marker for marker in action.pose_markers if marker.frame == frame]
            row = layout.row()
            if True:
                row.prop(speaker,"papagayo_phonemes",expand=True)
		
            row = layout.row()
            if len(timemarkers) and True:
            
                row.label(text="%s"%timemarkers[0].name,icon="MOD_MASK")
                row.prop(timemarkers[0],"frame",text="")
            else:
                timemarkers = [marker for marker in action.pose_markers if marker.frame > frame]
                if len(timemarkers):
                    row.label(text="NEXT %s"%timemarkers[0].name)
                else:
                    row.label("-")
        layout.prop(sce,"frame_current",emboss=True) 
    '''
    @classmethod 
    def poll(self,context):

        return context.object.data.sound is not None 
    '''
class SoundVisualiserOp(bpy.types.Operator):
    '''Bake sound to multiple freqs'''
    bl_idname = "speaker.visualise"
    bl_label = "Bake Sound to Multiple freq."
    channels = IntProperty(name="Channels",default=16,description="Number of frequencies to split",min=1,max=72)
    minf = FloatProperty(name="Min Freq",default=4.0,description="Minimum Freq",min=0,max=10000.0)
    maxf = FloatProperty(name="Max Freq",default=10000.0,description="Maximum Freq",min=100.0,max=1000000.0)
    attack = FloatProperty(name="Attack",default=0.005,description="Attack",min=0.001,max=0.1)
    action_name = StringProperty(name="Action Name",default="SoundAction")
    channel_name = StringProperty(name="Channel Name",default="Channel")
    
    release = FloatProperty(name="Release",default=0.2,description="Release")    
    threshold = FloatProperty(name="Threshold",default=0.005,description="Threshold")
    sthreshold = FloatProperty(name="SquareThreshold",default=0.005,description="Square Threshold")
    
    square = BoolProperty(name="Square",default=False,description="Square") 
    accumulate = BoolProperty(name="Accumulate",default=False,description="Accumulate")
    use_additive = BoolProperty(name="UseAdditive",default=False,description="Use Additive")           
    use_log = BoolProperty(name="Log Scale",default=True,description="Use Log scale for channels")
    log_base = IntProperty(name="log_base",default=16,description="log base to use",min=2,soft_min=2,soft_max=32,max=64)
    xaction = StringProperty(name="xaction",default="")


    crop = BoolProperty(default=False,description="Crop to new limits")
    TOL = FloatProperty(name="Max Val Limit",description="Discard channel values greater",default=100)
    driver_index = IntProperty(default=-1)
    contextspeakername = StringProperty(default="None")

    @classmethod
    def poll(cls, context):
        return True
        #return context.speaker is not None and context.speaker.sound is not None
    
    def invoke(self, context, event):
        #create a Sounds action


        if "SoundDrive" not in bpy.app.driver_namespace:
            InitSoundTools(context.scene)
        if self.driver_index > -1:
            bpy.types.SoundVisualiserPanel.driver_index = self.driver_index
            return {'RUNNING_MODAL'}
        if len(self.xaction):
            #print("ACTION %s"%self.xaction)
            soundaction = bpy.data.actions[self.xaction]
            if soundaction is not None:
                rna = soundaction["rna"]
                #print(rna)
                context.speaker["_RNA_UI"] = eval(rna)
            context.speaker.animation_data.action = soundaction
            
            return {'RUNNING_MODAL'}
        wm = context.window_manager
        
        if context.speaker.animation_data is not None and context.speaker.animation_data.action is not None:
            if "Channels" in context.speaker.animation_data.action:
                soundaction = context.speaker.animation_data.action
                self.channels = soundaction["Channels"]
                name = self.channel_name = soundaction["channel_name"]
                self.action_name = soundaction.name
                if self.crop:
                    #freqrange = eval(soundaction["needs_rebake"])
                    start = soundaction["start"]
                    end = soundaction["end"]
                    channel = "%s%d"%(name,start)
                    self.minf = context.speaker["_RNA_UI"][channel]["low"]
                    channel = "%s%d"%(name,end)
                    self.maxf = context.speaker["_RNA_UI"][channel]["high"]
                else:
                    self.minf = soundaction["minf"]
                    self.maxf = soundaction["maxf"]
                self.use_log = soundaction["use_log"]
        return wm.invoke_props_dialog(self)

    def draw(self,context):
        sce = context.scene
        layout = self.layout
        
        row = layout.row()
        row.label(context.speaker.sound.name)
        row = layout.row()
        row.prop(self,"action_name")
        row = layout.row()
        split = row.split(percentage=0.7)
        split.prop(self,"channel_name",text="Channel")
        split.prop(self,"channels")
        row = layout.row()
        split = row.split(percentage=0.40)
        split.prop(self,"use_log")
        
        split.prop(self,"log_base")
        box = layout.box()
        box.label("Bake Sound to fcurve",icon="IPO")
        box.prop(self,"minf")
        box.prop(self,"maxf")
        box.prop(self,"threshold")
        box.prop(self,"attack")
        box.prop(self,"release")
        box.prop(self,"use_additive",icon="PLUS")
        box.prop(self,"accumulate",icon="PLUS")
        row = box.row()
        split = row.split(percentage=0.20)
        split.prop(self,"square")
        split.prop(self,"sthreshold")
        layout.prop(self,"TOL")


    def execute(self, context):
        # set to frame 1
        rna = dict()
        action_rna = dict()
        save_frame = context.scene.frame_current
        context.scene.frame_set(1)
        
        if context.space_data.use_pin_id:
            #context.space_data.context = "OBJECT"
            speaker = context.space_data.pin_id
            #find the speaker object
            speakerobjs = [speakerobj for speakerobj in context.scene.objects if speakerobj.data == speaker]

            context.scene.objects.active = speakerobjs[0]
        else:
            speaker = context.object.data
            
        context.area.type = 'GRAPH_EDITOR' 
        soundaction = bpy.data.actions.new("%s"%(self.action_name))
        soundaction["Channels"] = self.channels
        soundaction["channel_name"] = self.channel_name
        soundaction["minf"] = self.minf
        soundaction["maxf"] = self.maxf
        soundaction["use_log"] = self.use_log
        soundaction["wavfile"] = speaker.sound.name

        #keep some UI stuff here too like the row height of each channel

        soundaction["row_height"] = 0.6

        
        #soundaction = fetch(bpy.data.actions,"Sounds",True)  #need to delete baked curves to use

        if not speaker.animation_data:
            speaker.animation_data_create()
        speaker.animation_data.action = soundaction
        if not soundaction:
            return{'CANCELLED'}
        #CHANGE CAPS TO OP PROPS   
        use_log = self.use_log
        MIN = 100
        MAX = -100    
        if use_log:
            if self.minf == 0:
                self.minf =1
            LOW = log(self.minf,self.log_base)
            HIGH = log(self.maxf,self.log_base)   
            RANGE = HIGH-LOW            
        else:    
            LOW = self.minf
            HIGH = self.maxf   
            RANGE = HIGH-LOW
            
        #file = context.scene["SoundFile"] # do some proper file check here.
        
        file = speaker.sound.filepath
        n = self.channels
        good = []
        for i in range(0,n):
            
            channel = '%s%d'%(self.channel_name,i)
            low = LOW + (i)*RANGE/n
            high = LOW + (i+1)*RANGE/n
            speaker[channel] = 0.0
            speaker.keyframe_insert('["%s"]'%channel)
            fcurve = soundaction.fcurves[i]
            if use_log:
                low = self.log_base**low
                high = self.log_base**high
            
            
            fcurve.select = True
            bpy.ops.graph.sound_bake(filepath=file, low=low, high=high, attack=self.attack, release=self.release, threshold=self.threshold, accumulate=self.accumulate, use_additive=self.use_additive, square=self.square, sthreshold=self.sthreshold, filter_sound=True)
            has_range,min_,max_ = getrange(fcurve,self.TOL)
            if min_ < MIN:
                MIN = min_
            elif max_ > MAX:
                MAX = max_
            
            if has_range:
               #flag it as dirty
               good.append(i)
            
            rna[channel] = {"min":0.0,"max":0.5, "description":"Frequency %s to %s (min:%.2f,max:%.2f)"%(f(low),f(high),min_,max_),"soft_min":min_, "soft_max":max_, "low":low, "high":high, "a":min_, "b":max_}
            fcurve.select = False
        
        for i in range(0,n):
            #channel = 'Channel%d'%i
            channel = '%s%d'%(self.channel_name,i)
            rna[channel]["min"] = MIN
            rna[channel]["max"] = MAX
            if True:  #USE SOFT MIN SLIDERS relative_scale
                rna[channel]["soft_min"] = MIN
                rna[channel]["soft_max"] = MAX
            
        soundaction["max"] = MAX
        soundaction["min"] = MIN
        soundaction["rna"] = str(rna)
        soundaction["start"] = min(good)
        clip = self.channels - 1
        soundaction["end"] = max(good)
        if len(good) < self.channels:
            soundaction["needs_rebake"] = "(%d,%d)"%(min(good),max(good))

        #soundaction["show_freq"] = False
        #action_rna["show_freq"] =  {"default":True}
        action_rna["row_height"] =  {"min":0.001,"max":1.0, "description":"Alter the row height","soft_min":0.0,"soft_max":1.0}
        action_rna["start"] =  {"min":0,"max":1.0, "description":"Clip Start","soft_min":0,"soft_max":clip}
        action_rna["end"] =  {"min":1,"max":clip, "description":"Clip End","soft_min":1,"soft_max":clip}
        speaker["_RNA_UI"] = rna
        soundaction["_RNA_UI"] = action_rna
        #speaker["Channels"] = n
        #speaker.keyframe_insert('["Channels"]')
        context.area.type = 'PROPERTIES' 
        context.scene.frame_set(save_frame)   
        speaker.vismode = "VISUAL"
        return{'FINISHED'}


class SoundVisMenu(bpy.types.Menu):
    bl_idname = "soundtest.menu"
    bl_label = "Select a Sound"
    vismode = "VISUAL"
    def draw(self, context):
        speaker = context.speaker
        #print("MENU:%s %s"%(speaker.vismode,SoundVisMenu.vismode))
        if SoundVisMenu.vismode in ["VISUAL","SOUND"]:
            actions = [action for action in bpy.data.actions if "wavfile" in action and action["wavfile"] == speaker.sound.name]
            for action in actions:
                self.layout.operator("speaker.visualise",text=action.name).xaction = action.name  
        elif SoundVisMenu.vismode == "DRIVERS":
            drivers = [driver for driver in speaker.animation_data.drivers ]
            for i,driver in enumerate(drivers):
                self.layout.operator("speaker.visualise",text="%s %s"%(driver.data_path,driver.driver.expression)).driver_index = i    


  
  
def showEqualiser(layout,speaker,action,info=True):
    if action:  
        minf = -1
        maxf = -1
        scale = action["row_height"]
        name = "Channel"
        if "Channels" in action:
            channels = action["Channels"]
            name = channel_name = action["channel_name"]
            minf = action["minf"]
            maxf = action["maxf"]
            start = action["start"]
            end = action["end"]
            
        else:
            channels = len(action.fcurves)
            minf = 0.0
            maxf = 0.0
            start = 0
            end = channels 
    
        if start >= end:
            end = start + 1
        i = start 

    box = layout
    if info:
        #box.scale_y = scale
        desc = "%d channels (%s to %s)"%(channels,f(minf),f(maxf))
        desc2 = "MIN VALUE %.2f MAX VALUE %.2f"%(action["min"],action["max"])
        '''
        row = box.row()
        if nla:
            row.label(icon="NLA")
        else:
            row.label(icon="ACTION")
            if not action.use_fake_user:
                row.prop(action,"use_fake_user",toggle=True,emboss=True,text="SAVE THIS?")
        row.enabled = not nla
        '''
        box.label(text=desc, icon="INFO")
        box.label(desc2)
        row = box.row()
        split = row.split(percentage=0.2)
        split.prop(action,"show_freq",emboss=True,text="",icon_only=True,icon="ARROW_LEFTRIGHT")
        split.prop(action,'["row_height"]',slider=True) 
        split.prop(action,'["start"]',slider=True) 
        split.prop(action,'["end"]',slider=True) 
        if (end + 1 - start) < channels:
            split.operator("speaker.visualise",text="CROP").crop = True
        
        #box.enabled = False
        #box.split(percentage=0.50)
        row = box.row()
        if "needs_rebake" in action:
            row.operator("speaker.visualise",text="REBAKE?",icon="ERROR").crop = True
            row = box.row()
            row.label(text="Not all frequencies baked")
    
    eqbox = box.box()
    eqbox.scale_x = scale
    eqbox.scale_y = scale
    sp = 0.2
    if action.show_freq: sp=0.01
    for i in range(start,end + 1):
        channel = "%s%d"%(channel_name,i)
        
        row = eqbox.row()
        #row.scale_y = scale
        #row.scale_x =  scale
        split = row.split(percentage=sp)
        split.label(f(speaker["_RNA_UI"][channel]["low"]))
        
        #col2.label(f(speaker["_RNA_UI"][channel]["low"]))
        rna = speaker["_RNA_UI"][channel]
        
        #print("%d smin %s"%(i,rna["soft_min"]))
        #print("smax %s"%rna["soft_max"])
        if float(rna["b"] - rna["a"]) < 0.0001:
            wbox = split.box()
            wbox.scale_y = 0.5
            wbox.scale_x = 0.5
            wbox.label("NO DATA IN THIS FREQUENCY")
        else:
            split.prop(speaker,'["%s"]'%channel,slider=True,event=True,emboss=True)
        #row.split(percentage=0.90)
        
        i += 1
    #eqbox.label(" "
    
    


def setcontextspeakerENUM(self,context):
    #print("Change mode to %s"%self.vismode)
    #print(context.object)
    SoundVisMenu.vismode = self.vismode
    if context.object is None or context.object.type != "SPEAKER":
        #print("Not a speaker")
        return(None)
    speaker = context.object.data
    
    #print(len(drivers))
    if self.vismode == "DRIVERS":
        if speaker.animation_data is None : speaker.animation_data_create() 
        drivers = speaker.animation_data.drivers
        if len(drivers) == 0:
            speaker["Driver0"] = 0.0
            driver = speaker.driver_add('["Driver0"]').driver
            bpy.types.SoundVisualiserPanel.driver_index = 0
            channel = "Channel0"
            var = driver.variables.new()
            var.type = "SINGLE_PROP"
            var.name = channel
            target = var.targets[0]
            target.id_type = "SPEAKER"
            target.id = speaker.id_data
            target.data_path = '["%s"]'%channel
            driver.expression = 'SoundDrive(%s)'%channel
            
def defaultPanels(regflag):
    if regflag:
        bpy.utils.register_class(DATA_PT_speaker)
        bpy.utils.register_class(DATA_PT_context_speaker)
        bpy.utils.register_class(DATA_PT_cone)
        bpy.utils.register_class(DATA_PT_distance)
        bpy.utils.register_class(DATA_PT_custom_props_speaker)
    else:
        bpy.utils.unregister_class(DATA_PT_speaker)
        bpy.utils.unregister_class(DATA_PT_context_speaker)
        bpy.utils.unregister_class(DATA_PT_cone)
        bpy.utils.unregister_class(DATA_PT_distance)
        bpy.utils.unregister_class(DATA_PT_custom_props_speaker)
    

     
def register():

    
    defaultPanels(False)
    bpy.types.Speaker.vismode = EnumProperty(items=(
                ("SOUND","SOUND","Edit sound properties"),
                ("VISUAL","VISUAL","Show sound visualiser"),
                ("DRIVERS","DRIVERS","Create Custom Driver(s)"),
                ("LIPSYNC","LIPSYNC","Add Lip sync to this wav")
                ),
                name="SoundDriver",
                default = "SOUND",
                description = "Vis to display, driver to build a driver",
                update=setcontextspeakerENUM,
                options={'HIDDEN'},
                
                )
    bpy.types.Action.show_freq = BoolProperty(default=True)
    bpy.utils.register_class(SoundVisualiserOp)
    bpy.utils.register_class(SoundVisualiserPanel)
    bpy.utils.register_class(SoundVisMenu)
    InitSoundTools(bpy.context.scene)

def unregister():
    defaultPanels(True)
    bpy.utils.unregister_class(SoundVisualiserOp)    
    bpy.utils.unregister_class(SoundVisualiserPanel)
    bpy.utils.unregister_class(SoundVisMenu)
    #bpy.app.handlers.load_post.remove(InitSoundTools)

